==> klayout-0.15.tar.gz: klayout-0.15/src/dbSaveLayoutOptions.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbSaveLayoutOptions.h"
#include "dbStream.h"
#include "tlClassRegistry.h"

namespace db
{

SaveLayoutOptions::SaveLayoutOptions ()
  : m_format ("GDS2"), m_all_layers (true), m_all_cells (true), m_dbu (0.0), m_scale_factor (1.0)
{
  // .. nothing yet ..
}

void 
SaveLayoutOptions::set_format (const std::string &format_name)
{
  m_format = format_name;
}

void 
SaveLayoutOptions::add_layer (unsigned int layer, const db::LayerProperties &props)
{
  m_all_layers = false;
  m_layers.insert (std::make_pair (layer, props));
}

void
SaveLayoutOptions::select_all_layers ()
{
  m_all_layers = true;
  m_layers.clear ();
}

void 
SaveLayoutOptions::add_cell (db::CellGraph::cell_index_type cell_index)
{
  m_all_cells = false;
  m_cells.insert (cell_index);
}

void
SaveLayoutOptions::select_all_cells ()
{
  m_all_cells = true;
  m_cells.clear ();
}

void 
SaveLayoutOptions::set_dbu (double dbu)
{
  m_dbu = dbu;
}

void 
SaveLayoutOptions::set_scale_factor (double f)
{
  m_scale_factor = f;
}

void 
SaveLayoutOptions::get_valid_layers (const db::Layout &layout, std::vector <std::pair <unsigned int, db::LayerProperties> > &layers, 
                                     bool (*valid_func) (const db::LayerProperties &, void *), void *client_data) const
{
  if (m_all_layers) {

    //  all layers selected -> report those which are valid
    for (unsigned int l = 0; l < layout.layers (); ++l) {
      if (layout.is_valid_layer (l)) {
        const db::LayerProperties &prop = layout.get_properties (l);
        if ((*valid_func) (prop, client_data)) {
          layers.push_back (std::make_pair (l, prop));
        }
      }
    }

  } else {

    typedef std::map<unsigned int, db::LayerProperties>::const_iterator iterator;

    //  just a subset -> report the ones that either have a valid override definition 
    //  or a valid layout definition
    for (iterator l = m_layers.begin (); l != m_layers.end (); ++l) {
      if (layout.is_valid_layer (l->first)) {
        if ((*valid_func) (l->second, client_data)) {
          layers.push_back (std::make_pair (l->first, l->second));
        } else {
          const LayerProperties &prop = layout.get_properties (l->first);
          if ((*valid_func) (prop, client_data)) {
            layers.push_back (std::make_pair (l->first, prop));
          }
        }
      }
    }

  }
}

void 
SaveLayoutOptions::get_cells (const db::Layout &layout, std::vector <db::CellGraph::cell_index_type> &cells) const
{
  if (m_all_cells) {
    for (db::Layout::bottom_up_const_iterator cell = layout.begin_bottom_up (); cell != layout.end_bottom_up (); ++cell) {
      cells.push_back (*cell);
    }
  } else {
    std::set <db::CellGraph::cell_index_type> selected;
    for (std::set <db::CellGraph::cell_index_type>::const_iterator c = m_cells.begin (); c != m_cells.end (); ++c) {
      selected.insert (*c);
      layout.cell (*c).collect_called_cells (selected);
    }
    cells.assign (selected.begin (), selected.end ());
  }
}

static bool 
match_format (const std::string &fn, const std::string &fmt)
{
  const char *fp = fmt.c_str ();
  while (*fp && *fp != '(') {
    ++fp;
  }
  while (*fp && *fp != ')') {
    if (*++fp == '*') {
      ++fp;
    }
    const char *fpp = fp;
    while (*fpp && *fpp != ' ' && *fpp != ')') {
      ++fpp;
    }
    if (fn.size () > (unsigned int) (fpp - fp) && strncmp (fn.c_str () + fn.size () - (fpp - fp), fp, fpp - fp) == 0) {
      return true;
    }
    fp = fpp;
    while (*fp == ' ') {
      ++fp;
    }
  }
  return false;
}

void 
SaveLayoutOptions::set_format_from_filename (const std::string &fn, bool &gzip)
{
  for (tl::Registrar<db::StreamFormatDeclaration>::iterator fmt = tl::Registrar<db::StreamFormatDeclaration>::begin (); fmt != tl::Registrar<db::StreamFormatDeclaration>::end (); ++fmt) {
    if (match_format (fn, fmt->file_format ())) {
      m_format = fmt->format_name ();
      break;
    }
  }
  if (match_format (fn, "(*.gz *.gzip *.GZ *.GZIP)")) {
    gzip = true;
  }
}

}

==> klayout-0.17.1.tar.gz: klayout-0.17.1/src/dbSaveLayoutOptions.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbSaveLayoutOptions.h"
#include "dbStream.h"
#include "tlClassRegistry.h"

namespace db
{

SaveLayoutOptions::SaveLayoutOptions ()
  : m_format ("GDS2"), m_all_layers (true), m_all_cells (true), m_dbu (0.0), m_scale_factor (1.0),
    mp_options (0)
{
  // .. nothing yet ..
}

SaveLayoutOptions::~SaveLayoutOptions ()
{
  set_specific_options (0);
}

SaveLayoutOptions::SaveLayoutOptions (const SaveLayoutOptions &d)
  : mp_options (0)
{
  operator= (d);
}

SaveLayoutOptions &
SaveLayoutOptions::operator= (const SaveLayoutOptions &d)
{
  if (this != &d) {

    m_format = d.m_format;
    m_layers = d.m_layers;
    m_cells = d.m_cells;
    m_all_layers = d.m_all_layers;
    m_all_cells = d.m_all_cells;
    m_dbu = d.m_dbu;
    m_scale_factor = d.m_scale_factor;

    if (d.get_specific_options ()) {
      set_specific_options (d.get_specific_options ()->clone ());
    } else {
      set_specific_options (0);
    }

  }
  return *this;
}

void 
SaveLayoutOptions::set_specific_options (const FormatSpecificWriterOptions &options)
{
  set_specific_options (options.clone ());
}

void 
SaveLayoutOptions::set_specific_options (const FormatSpecificWriterOptions *options)
{
  if (mp_options) {
    delete mp_options;
  }
  mp_options = options;
}

const FormatSpecificWriterOptions *
SaveLayoutOptions::get_specific_options () const
{
  return mp_options;
}

void 
SaveLayoutOptions::set_format (const std::string &format_name)
{
  m_format = format_name;
}

void 
SaveLayoutOptions::add_layer (unsigned int layer, const db::LayerProperties &props)
{
  m_all_layers = false;
  m_layers.insert (std::make_pair (layer, props));
}

void
SaveLayoutOptions::select_all_layers ()
{
  m_all_layers = true;
  m_layers.clear ();
}

void
SaveLayoutOptions::deselect_all_layers ()
{
  m_all_layers = false;
  m_layers.clear ();
}

void 
SaveLayoutOptions::add_cell (db::cell_index_type cell_index)
{
  m_all_cells = false;
  m_cells.insert (cell_index);
}

void
SaveLayoutOptions::select_all_cells ()
{
  m_all_cells = true;
  m_cells.clear ();
}

void 
SaveLayoutOptions::set_dbu (double dbu)
{
  m_dbu = dbu;
}

void 
SaveLayoutOptions::set_scale_factor (double f)
{
  m_scale_factor = f;
}

void 
SaveLayoutOptions::get_valid_layers (const db::Layout &layout, std::vector <std::pair <unsigned int, db::LayerProperties> > &layers, 
                                     bool (*valid_func) (const db::LayerProperties &, void *), void *client_data) const
{
  if (m_all_layers) {

    //  all layers selected -> report those which are valid
    for (unsigned int l = 0; l < layout.layers (); ++l) {
      if (layout.is_valid_layer (l)) {
        const db::LayerProperties &prop = layout.get_properties (l);
        if ((*valid_func) (prop, client_data)) {
          layers.push_back (std::make_pair (l, prop));
        }
      }
    }

  } else {

    typedef std::map<unsigned int, db::LayerProperties>::const_iterator iterator;

    //  just a subset -> report the ones that either have a valid override definition 
    //  or a valid layout definition
    for (iterator l = m_layers.begin (); l != m_layers.end (); ++l) {
      if (layout.is_valid_layer (l->first)) {
        if ((*valid_func) (l->second, client_data)) {
          layers.push_back (std::make_pair (l->first, l->second));
        } else {
          const LayerProperties &prop = layout.get_properties (l->first);
          if ((*valid_func) (prop, client_data)) {
            layers.push_back (std::make_pair (l->first, prop));
          }
        }
      }
    }

  }
}

void 
SaveLayoutOptions::get_cells (const db::Layout &layout, std::vector <db::cell_index_type> &cells) const
{
  if (m_all_cells) {
    for (db::Layout::bottom_up_const_iterator cell = layout.begin_bottom_up (); cell != layout.end_bottom_up (); ++cell) {
      cells.push_back (*cell);
    }
  } else {
    std::set <db::cell_index_type> selected;
    for (std::set <db::cell_index_type>::const_iterator c = m_cells.begin (); c != m_cells.end (); ++c) {
      selected.insert (*c);
      layout.cell (*c).collect_called_cells (selected);
    }
    cells.assign (selected.begin (), selected.end ());
  }
}

static bool 
match_format (const std::string &fn, const std::string &fmt)
{
  const char *fp = fmt.c_str ();
  while (*fp && *fp != '(') {
    ++fp;
  }
  while (*fp && *fp != ')') {
    if (*++fp == '*') {
      ++fp;
    }
    const char *fpp = fp;
    while (*fpp && *fpp != ' ' && *fpp != ')') {
      ++fpp;
    }
    if (fn.size () > (unsigned int) (fpp - fp) && strncmp (fn.c_str () + fn.size () - (fpp - fp), fp, fpp - fp) == 0) {
      return true;
    }
    fp = fpp;
    while (*fp == ' ') {
      ++fp;
    }
  }
  return false;
}

void 
SaveLayoutOptions::set_format_from_filename (const std::string &fn, bool &gzip)
{
  for (tl::Registrar<db::StreamFormatDeclaration>::iterator fmt = tl::Registrar<db::StreamFormatDeclaration>::begin (); fmt != tl::Registrar<db::StreamFormatDeclaration>::end (); ++fmt) {
    if (match_format (fn, fmt->file_format ())) {
      m_format = fmt->format_name ();
      break;
    }
  }
  if (match_format (fn, "(*.gz *.gzip *.GZ *.GZIP)")) {
    gzip = true;
  }
}

}
